#! /usr/bin/env python
# -*- coding: utf-8 -*-

# This file is part of Miscellaneous Container Templates.
#
# Copyright (c) 2007, 2008, 2009, 2010 Paul Pogonyshev.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.


import os
import sys
import re

from urlparse import urlunsplit


if os.path.dirname (sys.argv[0]):
    os.chdir (os.path.dirname (sys.argv[0]))

expected_file = os.path.join ('mct', 'impl', 'closed-hash-table.hpp')
if not os.path.isfile (expected_file):
    sys.exit ("%s: cannot find '%s', strange..." % (sys.argv[0], expected_file))


try:
    from docutils.core import publish_cmdline
except ImportError:
    sys.exit ("%s: docutils not found; get them from http://docutils.sourceforge.net/"
              % sys.argv[0])


input_file_name  = os.path.join ('doc', 'mct.txt')
output_file_name = os.path.join ('doc', 'mct.html')

sys.argv.extend (['--stylesheet-path=%s' % os.path.join ('doc', 'reST.css'),
                  '--embed-stylesheet',
                  '--no-source-link',
                  '--strip-comments',
                  '--language=en',
                  '--input-encoding=utf-8:strict',
                  '--output-encoding=utf-8:strict',
                  '--no-file-insertion',
                  '--field-name-limit=25',
                  input_file_name, output_file_name])


publish_cmdline (writer_name = 'html', description = 'Generate MCT documentation')


sys.stdout.write ("Documentation has been generated as file '%s'\n" % output_file_name)
sys.stdout.write ("Point your browser to %s\n"
                  % urlunsplit (('file', None, os.path.abspath (output_file_name), None, None)))



# Local variables:
# mode: python
# python-indent: 4
# indent-tabs-mode: nil
# fill-column: 90
# End:
