// Copyright (c) 2009, 2010 Paul Pogonyshev.
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.


#ifndef MCT__CLOSED_HASH_SET_HPP
#define MCT__CLOSED_HASH_SET_HPP


#include <mct/config.hpp>
#include <mct/impl/closed-hash-table.hpp>

#include MCT_HASH_HEADER


namespace mct
{

  namespace impl
  {

    template <typename Value>
    struct set_bucket_traits
    {
      typedef  Value             value_type;
      typedef  const value_type  assignable_value_type;
      typedef  value_type        key_type;

      static  const key_type&
      extract_key (const value_type& value)
      {  return value;  }
    };

    template <typename Value, bool keep_hashes>
    class set_bucket : public bucket_base <set_bucket_traits <Value>, keep_hashes>
    {
    public:

      bool
      mapped_equal (const set_bucket& that) const
      {  return true;  }
    };

    template <typename Value>
    struct set_buckets
    {
      typedef  set_bucket <Value, false>  without_hash;
      typedef  set_bucket <Value, true>   with_hash;
    };

  }


  template <typename Value,
            typename Hash        = MCT_HASH_NAMESPACE::hash <Value>,
            typename Equal       = std::equal_to <Value>,
            typename Allocator   = std::allocator <Value>,
            bool     keep_hashes = false>
  class closed_hash_set
    : public impl::closed_hash_table <impl::set_buckets <Value>,
                                      Hash, Equal, Allocator, keep_hashes>
  {
    typedef  impl::closed_hash_table <impl::set_buckets <Value>,
                                      Hash, Equal, Allocator, keep_hashes>
             table_type;

  public:

    explicit
    closed_hash_set (std::size_t      num_buckets = 0,
                     const Hash&      hash        = Hash (),
                     const Equal&     equal       = Equal (),
                     const Allocator& allocator   = Allocator ())
      : table_type (num_buckets, hash, equal, allocator)
    { }

    closed_hash_set (const closed_hash_set& that, const Allocator& allocator)
      : table_type (that, allocator)
    { }

    template <typename InputIterator>
    closed_hash_set (InputIterator first, InputIterator last,
                     std::size_t      num_buckets = 0,
                     const Hash&      hash        = Hash (),
                     const Equal&     equal       = Equal (),
                     const Allocator& allocator   = Allocator ())
      : table_type (first, last, num_buckets, hash, equal, allocator)
    { }


    closed_hash_set&
    operator= (const closed_hash_set& that)
    {
      return static_cast <closed_hash_set&> (table_type::operator= (that));
    }
  };


  template <typename Value, typename Hash, typename Equal, typename Allocator, bool keep_hashes>
  inline  void
  swap (closed_hash_set <Value, Hash, Equal, Allocator, keep_hashes>& set1,
        closed_hash_set <Value, Hash, Equal, Allocator, keep_hashes>& set2)
  {
    set1.swap (set2);
  }

}


#endif  // Multi-inclusion guard.


// Local variables:
// mode: c++
// c-basic-offset: 2
// indent-tabs-mode: nil
// fill-column: 90
// End:
